var _a;
import { EventEmitter } from 'events';
const signals = [
    'SIGHUP',
    'SIGINT',
    'SIGQUIT',
    'SIGILL',
    'SIGTRAP',
    'SIGABRT',
    'SIGBUS',
    'SIGFPE',
    'SIGUSR1',
    'SIGSEGV',
    'SIGUSR2',
    'SIGTERM',
];
class ShutdownError extends Error {
}
function isError(event) {
    return event instanceof Error;
}
function isEvent(event) {
    return typeof event === 'object' && event !== null && 'name' in event;
}
function getEventError(event) {
    if (isError(event))
        return event;
    if (isEvent(event)) {
        const error = new ShutdownError(event.message ?? event.name);
        error.name = event.name;
        error.code = event.code;
        error.detail = event.detail;
        error.stack = event.stack;
        return error;
    }
    return undefined;
}
function getEventCode(event) {
    if (typeof event === 'number')
        return event;
    if (typeof event !== 'string')
        return 1;
    const parsed = Number.parseInt(event);
    return !Number.isNaN(parsed) && parsed >= 0 ? parsed : 1;
}
export default class Shutdown {
    static register(key, func) {
        if (Shutdown.asyncFunctions[key])
            return;
        const wrappedFunc = async (eventOrExitCodeOrError) => {
            if (process.env.NODE_ENV === 'development')
                setTimeout(() => process.exit(1), 10000);
            try {
                await func(getEventError(eventOrExitCodeOrError));
            }
            catch (error) {
                Shutdown.emit('error', {
                    error: error instanceof Error ? error : new Error('unknown'),
                    register: key,
                });
            }
            process.exit(getEventCode(eventOrExitCodeOrError));
        };
        signals.forEach((evt) => process.on(evt, wrappedFunc));
        process.on('beforeExit', wrappedFunc);
        process.on('uncaughtException', wrappedFunc);
        process.on('unhandledRejection', wrappedFunc);
        Shutdown.asyncFunctions[key] = wrappedFunc;
    }
    static deregister(key) {
        if (!Shutdown.asyncFunctions[key])
            return;
        signals.forEach((evt) => process.off(evt, Shutdown.asyncFunctions[key]));
        process.off('beforeExit', Shutdown.asyncFunctions[key]);
        process.off('uncaughtException', Shutdown.asyncFunctions[key]);
        process.off('unhandledRejection', Shutdown.asyncFunctions[key]);
        delete Shutdown.asyncFunctions[key];
    }
    static emit(event, ...args) {
        this.eventEmitter.emit(event, ...args);
    }
    static on(event, listener) {
        this.eventEmitter.on(event, listener);
    }
    static off(event, listener) {
        this.eventEmitter.off(event, listener);
    }
}
_a = Shutdown;
Shutdown.asyncFunctions = {};
(() => {
    _a.eventEmitter = new EventEmitter();
})();
//# sourceMappingURL=index.js.map